/******************************************************************************
 * data_output.c
 * Federico Bareilles
 * 31/01/2003
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "tipper_red.h"

int print_header_flag = 0;

time_t day_to_seconds( float day )
{
  return (time_t) (( (day * 24.0 - 23.0) * 3600.0) + REFDAY);
}

struct tm *day_to_struct_tm(float day)
{
  time_t t = day_to_seconds(day);
  struct tm *tms = localtime(&t);

  return tms;
}

char *day_to_str( float day )
{
  static char str[20];
  struct tm *tms = day_to_struct_tm(day);

  sprintf(str,"%4d/%02d/%02d %02d:%02d:%02d",
	  tms->tm_year+1900, tms->tm_mon+1, tms->tm_mday,
	  tms->tm_hour, tms->tm_min, tms->tm_sec  );
  return str;
}


int print_header(FILE *fp_out, struct setup *s)
{
  struct instrument *t = s->tipper;

  fprintf(fp_out,"# INPUT FILE: %s\n", s->file_in);
  fprintf(fp_out,
  "# LINEAR: UNIT=#%1d REDANG=%2d GAINOFF=%6.3f ANGOF=%6.3f MIROFF=%6.3f\n",
	  t->unit, s->read_ang, GAINOFF, ANGOFF, MIROFF);
  fprintf(fp_out,"#            DAYOF = %8.3f  HROF = %7.3f\n",
	 0.0 , (float)s->sec_offset/(3600.0));
  fprintf(fp_out,"#            vref1_ADC = %6.1f  agrd_ADC = %4.1f\n",
	  t->vref1_ADC,t->agrd_ADC);   
  fprintf(fp_out,"#            ad_str: %s\n", t->ad_file);
  fprintf(fp_out,
	  "# TIME    TAU    CHI2   Tamb   Tins   TAU_S  W  RESID     5V       Vg      b\n");

  return 0;
}


int print_data(FILE *fp, struct setup *s)
{
  struct tdata *d;

  if(!print_header_flag){
    print_header(fp, s);
    print_header_flag = 1;
  }
  d=s->data;
  while( d != NULL ){
    if( d->f != NULL ){
      fprintf(fp,
	  "%8.3f %6.3f %7.3f %6.2f %6.2f %6.3f %2x %6.3f %8.2f %8.2f %7.3f",
	      d->date, d->f->tau, d->f->chi2f, d->t_abs /*d->t_adq */,
	      d->t_amp,
	      d->f->sigma_tau, d->error_flag, d->f->resid, d->v_comp,
	      d->v_gun, d->f->b);
    }else{
      fprintf(fp,
	      "%8.3f  #.###  ##.### %6.2f %6.2f ##.### %2x ##.### %8.2f %8.2f ##.###",
	      d->date, d->t_abs, d->t_amp, d->error_flag, d->v_comp, d->v_gun);
    }
    if(d->error_flag)
      fprintf(fp," #[%s]",day_to_str(d->date));
    if(d->error_flag & 1) /* Warning message in raw data file */
      fprintf(fp," %s",d->warning);
    fprintf(fp,"\n");

    d=d->next;
  }

return 0;
}


int print_tempe(FILE *fp, struct setup *s)
{
  struct tdata *d;

  if(!print_header_flag){
    fprintf(fp,
	    "# Time   Tadq   Tabs   Tgun   Tamp   Vcomp    Scomp Vgun     Sgun   Err\n");
           /*4199.000 287.36 279.49 282.65 281.05 4732.419 5.469 8790.390 2.137  0*/
    print_header_flag=1;
  }
  d=s->data;
  while(d != NULL){
    fprintf(fp,
	    "%8.3f %6.2f %6.2f %6.2f %6.2f %8.3f %5.3f %8.3f %5.3f %2x",
	    d->date, d->t_adq, d->t_abs, d->t_gun, d->t_amp, 
	    d->v_comp, d->v_comp_sigma,
	    d->v_gun, d->v_gun_sigma,
	    d->error_flag);
    if(d->error_flag)
      fprintf(fp," #[%s]",day_to_str(d->date));
    if(d->error_flag & 1) /* Warning message in raw data file */
      fprintf(fp," %s",d->warning);
    fprintf(fp,"\n");

    d=d->next;
  }

return 0;
}


int print_tsigma(FILE *fp, struct setup *s)
{
  struct tdata *d;

  if(!print_header_flag){
    fprintf(fp,
	    "# Time   Tau    Tau_S  Tadq   Err\n");
           /*4037.005  0.179  0.016 298.93  0*/
    print_header_flag=1;
  }

  d=s->data;
  while( d != NULL ){
    if( d->f != NULL ){
      fprintf(fp,
	      "%8.3f %6.3f %6.3f %6.2f %2x",
	      d->date, d->f->tau, d->f->sigma_tau, d->t_adq, d->error_flag);
    }else{
      fprintf(fp,
	      "#%8.3f ##.### ##.### %6.2f %2x",
	      d->date, d->t_adq, d->error_flag);
    }      
    if(d->error_flag)
      fprintf(fp," #[%s]",day_to_str(d->date));
    if(d->error_flag & 1) /* Warning message in raw data file */
      fprintf(fp," %s",d->warning);
    fprintf(fp,"\n");

    d=d->next;
  }

return 0;
}


int print_data_no_red(FILE *fp_out, struct setup *s)
{
  int i;
  struct tdata *d;
  
  if(!print_header_flag){
    fprintf(fp_out,"# Time   pm airms V_sky    Sky_S V_ref    Ref_S SOcf  Tadq   Vgun     S_Vg   Err\n");
    /*              4037.001 29 1.399 1656.014 1.519 1749.474 0.882 0.999 291.27 8648.982 1.911  0*/
    print_header_flag=1;
  }
  d=s->data;
  while(d != NULL){
    for(i=0; d->n > 2 && i < d->n; i++){
      struct tpoint *p = d->points+i;
      fprintf(fp_out,"%8.3f %2d %5.3f %8.3f %5.3f %8.3f %5.3f %5.3f %6.2f %8.3f %5.3f %2x",
	      d->date, p->pos, p->airmass, p->v_sky, p->v_sky_sigma,
	      p->v_ref, p->v_ref_sigma, s->spillover(p->airmass),
	      d->t_adq, d->v_gun, d->v_gun_sigma,
	      d->error_flag);
      if(d->error_flag & 1){
	fprintf(fp_out," %s[%s]",d->warning,day_to_str(d->date));
      }
      fprintf(fp_out,"\n");
    }
    d=d->next;
  }
  
    return 0;
}
