/******************************************************************************
 * minimos.c
 * Federico Bareilles
 * 08/02/2003
 *
 * Minimos cuadrados no lineales.
 *
 *****************************************************************************/

#define _POSIX_SOURCE 1         /* POSIX compliant source  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "matrix.h"
#include "minimos.h"


struct model *make_model( double (*modelo)())
{
  struct model *m;

  m = (struct model*) malloc(sizeof(struct model));
  m->modelo = modelo;
  m->parametros = 0;
  m->e = .000000001; /* .00000001 */
  m->deriv = m->current = NULL;
  m->xdata = m->ydata = m->sigma = NULL;
  m->VCV = NULL;

  return m;
}


int model_add_deriv(struct model *m, double (*func)())
{
  if(m->parametros == 0){
    m->current = m->deriv = 
      (struct derivada *) malloc(sizeof(struct derivada));
  }else{
    m->current = m->current->next = 
      (struct derivada *) malloc(sizeof(struct derivada));
  }
  m->current->func = func;
  m->parametros++;

  return m->parametros;
}


int fit_model(struct model *m, double *xx)
{
  int i,j;
  struct matrix *Cy, *iCy, *YV, *Vp, *Vpt;
  struct matrix *M1,*M2;
  struct derivada *deriv;
  double det;
  double res=1, res0;

  Cy = make_matrix(m->size,m->size);
  iCy = make_matrix(m->size,m->size);
  for(i=0; i< m->size; i++){
    for(j=0; j< m->size; j++){
      Cy->M[i][j] = iCy->M[i][j] = 0.0;
    }
    Cy->M[i][i] = *(m->sigma+i) * *(m->sigma+i);
    iCy->M[i][i] = 1.0 /Cy->M[i][i]; 
  }

  YV  = make_matrix(m->size,1);
  Vp  = make_matrix(m->size,m->parametros);
  Vpt = make_matrix(m->parametros,m->size);

  /* a partir de aca hay que iterar.... */
  do{
    res0=res;
    for(i=0; i< m->size; i++){
      YV->M[i][0] = m->ydata[i] - m->modelo(m->xdata[i], xx);
    }
    for(i=0; i< m->size; i++){
      deriv = m->deriv;
      for(j=0 && deriv->next != NULL; j < m->parametros; j++){
	Vp->M[i][j] = Vpt->M[j][i] = deriv->func(m->xdata[i], xx);
	deriv = deriv->next;
      }
    }
    /* Empiesa el metodo */
    M1 = prod_matrix(Vpt,iCy);
    M2 = prod_matrix(M1,Vp);
    det = det_matrix(M2);
    if( abs(det) != 0.0 ){
      /* printf("### Det %f\n",det); */
      if(m->VCV != NULL) free_matrix(m->VCV);
      m->VCV = inv_matrix(M2);
      /* print_matrix(m->VCV); */ /* Matrix de Varianza Covarianza */
      free_matrix(M2);
      M2 = prod_matrix(m->VCV,M1);
      free_matrix(M1);
      M1 = prod_matrix(M2,YV);
      free_matrix(M2);
      
      res = 0.0;
      for(i=0;i<m->parametros;i++){
	xx[i] += M1->M[i][0];
	res += M1->M[i][0] * M1->M[i][0];
      }
      free_matrix(M1);
    }else{
      /* fprintf(stderr,"Matriz singular\n"); */
      return MAT_SINGULAR;
    }
  }while( res > m->e );
  
  free_matrix(iCy);
  free_matrix(YV);
  free_matrix(Vp);
  free_matrix(Vpt);
  return 0;
}

